import customtkinter as ctk
import requests
from tkinter import messagebox
from datetime import datetime
from utils.api_requests import add_payment_method

SHOPPING = "#00c1ff"
DARK_BG = "#1f1f1f"
CARD_BG = "#2b2b2b"


def user_payments_frame(parent, switch_func, API_URL, token):
    """
    User payments page with a modern dark theme and improved layout.
    """
    # Main container frame
    frame = ctk.CTkFrame(parent, fg_color=DARK_BG)
    frame.token = token  # Store token as an attribute

    def update_token(new_token):
        """Update the token when it changes"""
        frame.token = new_token

    # Add the update_token method to the frame
    frame.update_token = update_token

    # --- TOP BAR ---
    top_bar = ctk.CTkFrame(frame, fg_color=SHOPPING, height=50)
    top_bar.pack(fill="x", side="top")

    top_label = ctk.CTkLabel(
        top_bar, text="My Payments", text_color="white", font=("Helvetica", 20, "bold")
    )
    top_label.pack(side="left", padx=25)

    def go_back():
        switch_func("dashboard")

    back_button = ctk.CTkButton(
        top_bar,
        text="← Back",
        fg_color="transparent",
        text_color="white",
        hover_color="#0096ff",
        command=go_back,
        width=80,
        height=35,
        font=("Helvetica", 14),
    )
    back_button.pack(side="right", padx=20, pady=7)

    # --- MAIN SECTION: Sidebar + Content ---
    main_section = ctk.CTkFrame(frame, fg_color="transparent")
    main_section.pack(fill="both", expand=True)

    # LEFT SIDEBAR
    sidebar_frame = ctk.CTkFrame(
        main_section, width=250, fg_color=CARD_BG, corner_radius=15
    )
    sidebar_frame.pack(side="left", fill="y", padx=20, pady=20)
    sidebar_frame.pack_propagate(False)

    sidebar_title = ctk.CTkLabel(
        sidebar_frame, text="Menu", font=("Helvetica", 18, "bold"), text_color=SHOPPING
    )
    sidebar_title.pack(pady=(20, 10))

    def create_nav_button(text, command=None, is_active=False):
        return ctk.CTkButton(
            sidebar_frame,
            text=text,
            fg_color="transparent",
            text_color="white",
            hover_color="#3b3b3b",
            command=command,
            height=40,
            font=("Helvetica", 14),
            state="disabled" if is_active else "normal",
        )

    nav_dashboard = create_nav_button("Dashboard", go_back)
    nav_dashboard.pack(fill="x", padx=15, pady=5)

    def open_profile():
        switch_func("user_details")

    nav_profile = create_nav_button("My Profile", open_profile)
    nav_profile.pack(fill="x", padx=15, pady=5)

    def open_orders():
        switch_func("user_orders")

    nav_orders = create_nav_button("My Orders", open_orders)
    nav_orders.pack(fill="x", padx=15, pady=5)

    nav_payments = create_nav_button("My Payments", is_active=True)
    nav_payments.pack(fill="x", padx=15, pady=5)

    # RIGHT CONTENT
    content_frame = ctk.CTkFrame(main_section, fg_color=CARD_BG, corner_radius=15)
    content_frame.pack(side="left", fill="both", expand=True, padx=(0, 20), pady=20)

    # Header
    header_frame = ctk.CTkFrame(content_frame, fg_color="transparent")
    header_frame.pack(fill="x", padx=30, pady=20)

    title_label = ctk.CTkLabel(
        header_frame,
        text="Payment Methods",
        font=("Helvetica", 24, "bold"),
        text_color=SHOPPING,
    )
    title_label.pack(anchor="w")

    subtitle_label = ctk.CTkLabel(
        header_frame,
        text="Add and manage your payment methods",
        font=("Helvetica", 14),
        text_color="gray",
    )
    subtitle_label.pack(anchor="w", pady=(5, 0))

    # Main Content
    main_content = ctk.CTkFrame(content_frame, fg_color="transparent")
    main_content.pack(fill="both", expand=True, padx=30, pady=(0, 20))

    # Payment Form
    form_frame = ctk.CTkFrame(main_content, fg_color="transparent")
    form_frame.pack(fill="x", pady=(0, 20))

    # Card Number Field
    card_label = ctk.CTkLabel(
        form_frame, text="Card Number", font=("Helvetica", 14), text_color="white"
    )
    card_label.pack(anchor="w", pady=(0, 5))

    card_entry = ctk.CTkEntry(
        form_frame,
        placeholder_text="Enter your card number",
        height=40,
        corner_radius=8,
    )
    card_entry.pack(fill="x", pady=(0, 15))

    # Card Type Selection
    card_type_label = ctk.CTkLabel(
        form_frame, text="Card Type", font=("Helvetica", 14), text_color="white"
    )
    card_type_label.pack(anchor="w", pady=(0, 5))

    card_type_var = ctk.StringVar(value="visa")
    card_type_options = ["Visa", "Mastercard", "PayPal"]

    card_type_dropdown = ctk.CTkOptionMenu(
        form_frame,
        values=card_type_options,
        variable=card_type_var,
        width=200,
        height=40,
        corner_radius=8,
        fg_color="#3b3b3b",
        button_color=SHOPPING,
        button_hover_color="#0096ff",
        dropdown_fg_color="#3b3b3b",
        font=("Helvetica", 14),
    )
    card_type_dropdown.pack(anchor="w", pady=(0, 15))

    # Expiration Date
    exp_label = ctk.CTkLabel(
        form_frame,
        text="Expiration Date (MM/YY)",
        font=("Helvetica", 14),
        text_color="white",
    )
    exp_label.pack(anchor="w", pady=(0, 5))

    exp_frame = ctk.CTkFrame(form_frame, fg_color="transparent")
    exp_frame.pack(fill="x", pady=(0, 15))

    month_entry = ctk.CTkEntry(
        exp_frame, placeholder_text="MM", width=60, height=40, corner_radius=8
    )
    month_entry.pack(side="left", padx=(0, 5))

    slash_label = ctk.CTkLabel(
        exp_frame, text="/", font=("Helvetica", 14), text_color="white"
    )
    slash_label.pack(side="left", padx=5)

    year_entry = ctk.CTkEntry(
        exp_frame, placeholder_text="YY", width=60, height=40, corner_radius=8
    )
    year_entry.pack(side="left", padx=5)

    # Security Code
    cvv_label = ctk.CTkLabel(
        form_frame, text="Security Code", font=("Helvetica", 14), text_color="white"
    )
    cvv_label.pack(anchor="w", pady=(0, 5))

    cvv_entry = ctk.CTkEntry(
        form_frame,
        placeholder_text="CVV",
        width=100,
        height=40,
        corner_radius=8,
        show="*",
    )
    cvv_entry.pack(anchor="w")

    def validate_card_number(event=None):
        value = card_entry.get().replace(" ", "")
        if not value.isdigit():
            card_entry.delete(0, "end")
            card_entry.insert(0, "".join(c for c in value if c.isdigit()))
        if len(value) > 16:
            card_entry.delete(16, "end")

    def validate_month(event=None):
        value = month_entry.get()
        if not value.isdigit():
            month_entry.delete(0, "end")
            return
        if len(value) > 2:
            month_entry.delete(2, "end")
        if value and int(value) > 12:
            month_entry.delete(0, "end")
            month_entry.insert(0, "12")

    def validate_year(event=None):
        value = year_entry.get()
        if not value.isdigit():
            year_entry.delete(0, "end")
            return
        if len(value) > 2:
            year_entry.delete(2, "end")
        # Only validate the range if we have 2 digits
        if len(value) == 2 and (int(value) < 25 or int(value) > 99):
            messagebox.showerror("Error", "Please input the correct year")
            year_entry.delete(0, "end")

    def validate_cvv(event=None):
        value = cvv_entry.get()
        if not value.isdigit():
            cvv_entry.delete(0, "end")
            cvv_entry.insert(0, "".join(c for c in value if c.isdigit()))
        if len(value) > 3:
            cvv_entry.delete(3, "end")

    # Bind validation functions
    card_entry.bind("<KeyRelease>", validate_card_number)
    month_entry.bind("<KeyRelease>", validate_month)
    year_entry.bind("<KeyRelease>", validate_year)
    cvv_entry.bind("<KeyRelease>", validate_cvv)

    def save_payment():
        # Validate all fields
        card_num = card_entry.get().strip()
        month = month_entry.get().strip()
        year = year_entry.get().strip()
        cvv = cvv_entry.get().strip()
        card_type = card_type_var.get().lower()  # Convert to lowercase to match backend expectations

        if not all([card_num, month, year, cvv]):
            messagebox.showerror("Error", "Please fill in all fields")
            return

        if len(card_num) < 12:
            messagebox.showerror("Error", "Invalid card number")
            return

        if not (1 <= int(month) <= 12):
            messagebox.showerror("Error", "Invalid month")
            return

        if not (25 <= int(year) <= 99):
            messagebox.showerror("Error", "Invalid year")
            return

        if len(cvv) != 3:
            messagebox.showerror("Error", "Invalid security code")
            return

        # Create payload
        payload = {
            "payment_method": card_type,
            "card_number": card_num,
            "expiry_date": f"{month}/{year}",
            "cvv": cvv
        }

        # Use the API function to save payment
        status_code, response = add_payment_method(API_URL, frame.token, payload)
        
        if status_code == 200:
            messagebox.showinfo("Success", "Payment method added successfully!")
            # Clear form
            card_entry.delete(0, "end")
            month_entry.delete(0, "end")
            year_entry.delete(0, "end")
            cvv_entry.delete(0, "end")
            card_type_var.set("Visa")  # Reset to default option
        else:
            error_msg = "Failed to add payment method"
            if isinstance(response, dict) and "detail" in response:
                error_msg = response["detail"]
            messagebox.showerror("Error", error_msg)

    # Save Button
    save_button = ctk.CTkButton(
        form_frame,
        text="Save Payment Method",
        command=save_payment,
        height=45,
        corner_radius=8,
        font=("Helvetica", 14, "bold"),
        fg_color=SHOPPING,
        hover_color="#0096ff",
    )
    save_button.pack(fill="x", pady=(30, 0))

    return frame
